using System;
using System.IO;
using System.Text;
using nToolbox.IO;
using NUnit.Framework;

namespace nToolbox.Core.Test
{
    [TestFixture]
    public class TestFileHelper : Assert
    {
        static string TempFolder = Path.Combine(Path.GetTempPath(), "nToolbox.Core.IO.Tests");
        private string _codeFileName = TempFolder + Path.DirectorySeparatorChar + "AFile.txt";

        [SetUp]
        public void SetUp()
        {
            if (!Directory.Exists(TempFolder))
                Directory.CreateDirectory(TempFolder);

            if (!File.Exists(_codeFileName))
            {
                File.Create(_codeFileName).Close();
                StreamWriter sr = File.AppendText(_codeFileName);
                sr.Write("test file");
                sr.Close();
                sr.Dispose();
            }
        }

        [TearDown]
        public void TearDown()
        {
            if (Directory.Exists(TempFolder))
                Directory.Delete(TempFolder, true);
        }

        [Test]
        public void TestGetFileText()
        {
            Assert.AreEqual("test file", FileHelper.GetFileText(_codeFileName));
        }

        [Test]
        public void TestCreateToFile()
        {
            string strFile = TempFolder + Path.DirectorySeparatorChar + "BFile.txt";
            FileHelper.CreateToFile(strFile, "another test");
            Assert.AreEqual("another test", FileHelper.GetFileText(strFile));
        }

        [Test]
        public void TestUpdateFileText()
        {
            FileHelper.UpdateFileText(_codeFileName, "file", "file with some more text");
            Assert.AreEqual("test file with some more text", FileHelper.GetFileText(_codeFileName));
        }

        [Test]
        public void TestWriteToFile()
        {
            FileHelper.WriteToFile(_codeFileName, "updated text");
            Assert.AreEqual("updated text", FileHelper.GetFileText(_codeFileName));
        }

        [Test]
        public void TestFromByteCount()
        {            
            Assert.AreEqual("136.17 KB", FileHelper.FromByteCount(139434));
            Assert.AreEqual("9 B", FileHelper.FromByteCount(new FileInfo(_codeFileName).Length));
        }
        
    }
}
