using System;
using NUnit.Framework;

namespace nToolbox.Core.Test
{
    [TestFixture]
    public class TestDateTimeHelper
    {
        [Test]
        public void TestDateInPast()
        {
            Assert.IsFalse(DateTimeHelper.IsDateInPast(DateTime.Now));
            Assert.IsFalse(DateTimeHelper.IsDateInPast(DateTime.Now.AddDays(1)));
            Assert.IsFalse(DateTimeHelper.IsDateInPast(DateTime.Now.AddMonths(1)));
            Assert.IsFalse(DateTimeHelper.IsDateInPast(DateTime.Now.AddYears(1)));
            Assert.IsFalse(DateTimeHelper.IsDateInPast(new DateTime(2199, 12, 31)));
            Assert.IsTrue(DateTimeHelper.IsDateInPast(DateTime.Now.AddDays(-1)));
            Assert.IsTrue(DateTimeHelper.IsDateInPast(DateTime.Now.AddMonths(-1)));
            Assert.IsTrue(DateTimeHelper.IsDateInPast(DateTime.Now.AddYears(-1)));
            Assert.IsTrue(DateTimeHelper.IsDateInPast(new DateTime(1999, 12, 31)));
        }

        [Test]
        public void TestDateInFuture()
        {
            Assert.IsFalse(DateTimeHelper.IsDateInFuture(DateTime.Now));
            Assert.IsTrue(DateTimeHelper.IsDateInFuture(DateTime.Now.AddDays(1)));
            Assert.IsTrue(DateTimeHelper.IsDateInFuture(DateTime.Now.AddMonths(1)));
            Assert.IsTrue(DateTimeHelper.IsDateInFuture(DateTime.Now.AddYears(1)));
            Assert.IsTrue(DateTimeHelper.IsDateInFuture(new DateTime(2199, 12, 31)));
            Assert.IsFalse(DateTimeHelper.IsDateInFuture(DateTime.Now.AddDays(-1)));
            Assert.IsFalse(DateTimeHelper.IsDateInFuture(DateTime.Now.AddMonths(-1)));
            Assert.IsFalse(DateTimeHelper.IsDateInFuture(DateTime.Now.AddYears(-1)));
            Assert.IsFalse(DateTimeHelper.IsDateInFuture(new DateTime(1999, 12, 31)));
        }

        [Test]
        public void TestEarliestDate()
        {
            DateTime dt1 = new DateTime(2005, 1, 1);
            DateTime dt2 = new DateTime(2006, 1, 1);
            Assert.AreEqual(dt1, DateTimeHelper.EarliestDate(dt1, dt2));
            Assert.AreEqual(dt1, DateTimeHelper.EarliestDate(dt2, dt1));

            dt1 = new DateTime(2006, 12, 1);
            dt2 = new DateTime(2006, 11, 30);
            Assert.AreEqual(dt2, DateTimeHelper.EarliestDate(dt1, dt2));
            Assert.AreEqual(dt2, DateTimeHelper.EarliestDate(dt2, dt1));

            dt1 = DateTime.Now;
            dt2 = DateTime.Now;
            Assert.AreEqual(dt1, DateTimeHelper.EarliestDate(dt1, dt2));
            Assert.AreEqual(dt1, DateTimeHelper.EarliestDate(dt2, dt1));
        }

        [Test]
        public void TestLastestDate()
        {
            DateTime dt1 = new DateTime(2005, 1, 1);
            DateTime dt2 = new DateTime(2006, 1, 1);
            Assert.AreEqual(dt2, DateTimeHelper.LatestDate(dt1, dt2));
            Assert.AreEqual(dt2, DateTimeHelper.LatestDate(dt2, dt1));

            dt1 = new DateTime(2006, 12, 1);
            dt2 = new DateTime(2006, 11, 30);
            Assert.AreEqual(dt1, DateTimeHelper.LatestDate(dt1, dt2));
            Assert.AreEqual(dt1, DateTimeHelper.LatestDate(dt2, dt1));

            dt1 = DateTime.Now;
            dt2 = DateTime.Now;
            Assert.AreEqual(dt1, DateTimeHelper.LatestDate(dt1, dt2));
            Assert.AreEqual(dt1, DateTimeHelper.LatestDate(dt2, dt1));
        }

        [Test]
        public void TestDatesAreInSameMonth()
        {
            DateTime dt1 = new DateTime(2007, 3, 1);
            DateTime dt2 = new DateTime(2007, 3, 15);
            Assert.IsTrue(DateTimeHelper.DatesAreInSameMonth(dt1, dt2));

            dt2 = new DateTime(2007, 2, 1);
            Assert.IsFalse(DateTimeHelper.DatesAreInSameMonth(dt1, dt2));

            dt2 = new DateTime(2006, 3, 1);
            Assert.IsTrue(DateTimeHelper.DatesAreInSameMonth(dt1, dt2));

        }

        [Test]
        public void TestDatesAreInSameYear()
        {
            DateTime dt1 = new DateTime(2007, 3, 1);
            DateTime dt2 = new DateTime(2007, 3, 15);
            Assert.IsTrue(DateTimeHelper.DatesAreInSameYear(dt1, dt2));

            dt2 = new DateTime(2007, 2, 1);
            Assert.IsTrue(DateTimeHelper.DatesAreInSameYear(dt1, dt2));

            dt2 = new DateTime(2006, 3, 1);
            Assert.IsFalse(DateTimeHelper.DatesAreInSameYear(dt1, dt2));

        }

        [Test]
        public void TestDatesAreInSameMonthAndYear()
        {
            DateTime dt1 = new DateTime(2007, 3, 1);
            DateTime dt2 = new DateTime(2007, 3, 15);
            Assert.IsTrue(DateTimeHelper.DatesAreInSameMonthAndYear(dt1, dt2));

            dt2 = new DateTime(2007, 2, 1);
            Assert.IsFalse(DateTimeHelper.DatesAreInSameMonthAndYear(dt1, dt2));

            dt2 = new DateTime(2006, 3, 1);
            Assert.IsFalse(DateTimeHelper.DatesAreInSameMonthAndYear(dt1, dt2));
            
            
            

        }

        [Test]
        public void TestDaysLeftInMonth()
        {
            Assert.AreEqual(30, DateTimeHelper.DaysRemainingInMonth(new DateTime(2007, 1, 1)));
            Assert.AreEqual(1, DateTimeHelper.DaysRemainingInMonth(new DateTime(2007, 1, 30)));
            Assert.AreEqual(0, DateTimeHelper.DaysRemainingInMonth(new DateTime(2007, 1, 31)));
            Assert.AreEqual(1, DateTimeHelper.DaysRemainingInMonth(new DateTime(2008, 2, 28)));
            Assert.AreEqual(0, DateTimeHelper.DaysRemainingInMonth(new DateTime(2009, 2, 28)));

            Assert.AreEqual(DateTimeHelper.DaysRemainingInMonth(), DateTimeHelper.DaysRemainingInMonth(DateTime.Now));
        }

        [Test]
        public void TestDaysLeftInYear()
        {
            Assert.AreEqual(364, DateTimeHelper.DaysRemainingInYear(new DateTime(2007, 1, 1)));
            Assert.AreEqual(365, DateTimeHelper.DaysRemainingInYear(new DateTime(2008, 1, 1)));
            Assert.AreEqual(6, DateTimeHelper.DaysRemainingInYear(new DateTime(2007, 12, 25)));
            Assert.AreEqual(6, DateTimeHelper.DaysRemainingInYear(new DateTime(2008, 12, 25)));

            Assert.AreEqual(DateTimeHelper.DaysRemainingInYear(), DateTimeHelper.DaysRemainingInYear(DateTime.Now));
        }

        [Test]
        public void TestFirstOccurenceOfDayInMonth()
        {
            int year = 2007;
            int month = 3;

            Assert.AreEqual(5, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Monday, month, year));
            Assert.AreEqual(6, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Tuesday, month, year));
            Assert.AreEqual(7, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Wednesday, month, year));
            Assert.AreEqual(1, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Thursday, month, year));
            Assert.AreEqual(2, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Friday, month, year));
            Assert.AreEqual(3, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Saturday, month, year));
            Assert.AreEqual(4, DateTimeHelper.FirstOccurenceOfDayInMonth(DayOfWeek.Sunday, month, year));
        }

        [Test]
        public void TestLastOccurenceOfDayInMonth()
        {
            int year = 2007;
            int month = 3;

            Assert.AreEqual(26, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Monday, month, year));
            Assert.AreEqual(27, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Tuesday, month, year));
            Assert.AreEqual(28, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Wednesday, month, year));
            Assert.AreEqual(29, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Thursday, month, year));
            Assert.AreEqual(30, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Friday, month, year));
            Assert.AreEqual(31, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Saturday, month, year));
            Assert.AreEqual(25, DateTimeHelper.LastOccurenceOfDayInMonth(DayOfWeek.Sunday, month, year));
        }

        [Test]
        public void TestConvertDayOfYearToDate()
        {
            Assert.AreEqual(new DateTime(2007, 1, 1), DateTimeHelper.ConvertDayOfYearToDate(1, 2007));
            Assert.AreEqual(new DateTime(2007, 12, 31), DateTimeHelper.ConvertDayOfYearToDate(365, 2007));
            Assert.AreEqual(new DateTime(2007, 2, 1), DateTimeHelper.ConvertDayOfYearToDate(32, 2007));
        }

        [Test]
        public void TestDiffToHourMinuteSeconds()
        {
            Assert.AreEqual("00:00:30", DateTimeHelper.DiffToHourMinuteSeconds(Convert.ToDateTime("7/1/2007 5:40:00 AM"), Convert.ToDateTime("7/1/2007 5:40:30 AM")));
            Assert.AreEqual("00:12:30", DateTimeHelper.DiffToHourMinuteSeconds(Convert.ToDateTime("7/1/2007 5:40:00 AM"), Convert.ToDateTime("7/1/2007 5:52:30 AM")));
            Assert.AreEqual("14:12:30", DateTimeHelper.DiffToHourMinuteSeconds(Convert.ToDateTime("7/1/2007 5:40:00 AM"), Convert.ToDateTime("7/1/2007 7:52:30 PM")));
        }

        [Test]
        public void TestDaysAgo()
        {
            TimeSpan t = new TimeSpan(30, 0, 0, 0);
            DateTime dtVerify = DateTime.Now.Subtract(t);

            DateTime dt = DateTimeHelper.DaysAgo(30);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestDaysFromNow()
        {
            TimeSpan t = new TimeSpan(30, 0, 0, 0);
            DateTime dtVerify = DateTime.Now.Add(t);

            DateTime dt = DateTimeHelper.DaysFromNow(30);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestHoursAgo()
        {
            TimeSpan t = new TimeSpan(0, 15, 0, 0);
            DateTime dtVerify = DateTime.Now.Subtract(t);

            DateTime dt = DateTimeHelper.HoursAgo(15);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestHoursFromNow()
        {
            TimeSpan t = new TimeSpan(0, 15, 0, 0);
            DateTime dtVerify = DateTime.Now.Add(t);

            DateTime dt = DateTimeHelper.HoursFromNow(15);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestMinutesAgo()
        {
            TimeSpan t = new TimeSpan(0, 45, 0);
            DateTime dtVerify = DateTime.Now.Subtract(t);

            DateTime dt = DateTimeHelper.MinutesAgo(45);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestMinutesFromNow()
        {
            TimeSpan t = new TimeSpan(0, 45, 0);
            DateTime dtVerify = DateTime.Now.Add(t);

            DateTime dt = DateTimeHelper.MinutesFromNow(45);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestSecondsAgo()
        {
            TimeSpan t = new TimeSpan(0, 0, 35);
            DateTime dtVerify = DateTime.Now.Subtract(t);

            DateTime dt = DateTimeHelper.SecondsAgo(35);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestSecondsFromNow()
        {
            TimeSpan t = new TimeSpan(0, 0, 35);
            DateTime dtVerify = DateTime.Now.Add(t);

            DateTime dt = DateTimeHelper.SecondsFromNow(35);
            Assert.IsTrue(dtVerify.Date == dt.Date);
        }

        [Test]
        public void TestDiff()
        {
            Assert.AreEqual(new TimeSpan(4, 0, 0, 0), DateTimeHelper.Diff(new DateTime(2007, 7, 5), new DateTime(2007, 7, 1)));
            Assert.AreEqual(new TimeSpan(5, 5, 5, 5), DateTimeHelper.Diff(new DateTime(2007, 7, 6, 7, 18, 47), new DateTime(2007, 7, 1, 2, 13, 42)));            
        }

        [Test]
        public void TestDiffDays()
        {
            Assert.AreEqual(4, DateTimeHelper.DiffDays(new DateTime(2007, 7, 5), new DateTime(2007, 7, 1)));
            Assert.AreEqual(4, DateTimeHelper.DiffDays("7/5/2007", "7/1/2007"));
        }

        [Test]
        public void TestDiffHours()
        {
            Assert.AreEqual(96, DateTimeHelper.DiffHours("7/5/2007", "7/1/2007"));
            Assert.AreEqual(125.08333333333333, DateTimeHelper.DiffHours("7/6/2007 7:18:47", "7/1/2007 2:13:47"));
            Assert.AreEqual(125.08333333333333, DateTimeHelper.DiffHours(new DateTime(2007, 7, 6, 7, 18, 47), new DateTime(2007, 7, 1, 2, 13, 47)));
            Assert.AreEqual(5, DateTimeHelper.DiffHours(new DateTime(2007, 7, 6, 7, 18, 0), new DateTime(2007, 7, 6, 2, 18, 0)));
        }

        [Test]
        public void TestDiffMinutes()
        {
            Assert.AreEqual(5760, DateTimeHelper.DiffMinutes("7/5/2007", "7/1/2007"));
            Assert.AreEqual(7505, DateTimeHelper.DiffMinutes("7/6/2007 7:18:47", "7/1/2007 2:13:47"));
            Assert.AreEqual(7505, DateTimeHelper.DiffMinutes(new DateTime(2007, 7, 6, 7, 18, 47), new DateTime(2007, 7, 1, 2, 13, 47)));
            Assert.AreEqual(300, DateTimeHelper.DiffMinutes(new DateTime(2007, 7, 6, 7, 18, 0), new DateTime(2007, 7, 6, 2, 18, 0)));
        }

        [Test]
        public void TestReadableDiff()
        {
            DateTime dtStartDate = Convert.ToDateTime("7/1/2007 5:00:00 AM");
            Assert.AreEqual("1 second ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddSeconds(1)));
            Assert.AreEqual("30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddSeconds(30)));
            Assert.AreEqual("1 minute ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(1)));
            Assert.AreEqual("2 minutes, 1 second ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(2).AddSeconds(1)));
            Assert.AreEqual("1 minute, 30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(1).AddSeconds(30)));
            Assert.AreEqual("1 minute, 1 second ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(1).AddSeconds(1)));
            Assert.AreEqual("2 minutes, 30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(2).AddSeconds(30)));
            Assert.AreEqual("4 hours ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddHours(4)));
            Assert.AreEqual("1 hour ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddHours(1)));
            Assert.AreEqual("4 hours, 30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddSeconds(30).AddHours(4)));
            Assert.AreEqual("4 hours, 2 minutes, 30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(2).AddSeconds(30).AddHours(4)));
            Assert.AreEqual("10 days, 4 hours, 2 minutes, 30 seconds ago", DateTimeHelper.ReadableDiff(dtStartDate, dtStartDate.AddMinutes(2).AddSeconds(30).AddHours(4).AddDays(10)));
        }

        [Test]
        public void TestCountWeekdays()
        {
            DateTime dtStart = Convert.ToDateTime("7/1/2007");
            Assert.AreEqual(12, DateTimeHelper.CountWeekdays(dtStart, dtStart.AddDays(14)));
        }

        [Test]
        public void TestCountWeekends()
        {
            DateTime dtStart = Convert.ToDateTime("7/1/2007");
            Assert.AreEqual(2, DateTimeHelper.CountWeekends(dtStart, dtStart.AddDays(14)));
        }

        [Test]
        public void TestIsDate()
        {
            Assert.AreEqual(true, DateTimeHelper.IsDate("7/1/2007"));
            Assert.AreEqual(true, DateTimeHelper.IsDate(Convert.ToDateTime("7/1/2007")));
            Assert.AreEqual(false, DateTimeHelper.IsDate("Not a Date"));
        }

        [Test]
        public void TestIsWeekDay()
        {
            Assert.AreEqual(true, DateTimeHelper.IsWeekDay(Convert.ToDateTime("7/1/2007")));
            Assert.AreEqual(false, DateTimeHelper.IsWeekDay(Convert.ToDateTime("7/7/2007")));
        }

        [Test]
        public void TestIsWeekEnd()
        {
            Assert.AreEqual(false, DateTimeHelper.IsWeekEnd(Convert.ToDateTime("7/1/2007")));
            Assert.AreEqual(true, DateTimeHelper.IsWeekEnd(Convert.ToDateTime("7/7/2007")));
        }

        [Test]
        public void TestTimeDiff()
        {
            DateTime dtStartDate = Convert.ToDateTime("7/1/2007 5:00:00 AM");
            Assert.AreEqual("1 second ", DateTimeHelper.TimeDiff(dtStartDate, dtStartDate.AddSeconds(1)));
            Assert.AreEqual("30 seconds ", DateTimeHelper.TimeDiff(dtStartDate, dtStartDate.AddSeconds(30)));
            Assert.AreEqual("1 minute 0 seconds ", DateTimeHelper.TimeDiff(dtStartDate, dtStartDate.AddMinutes(1)));            
        }

        [Test]
        public void TestGetFormattedMonthAndDay()
        {            
            Assert.AreEqual("July 1st", DateTimeHelper.GetFormattedMonthAndDay(Convert.ToDateTime("7/1/2007")));
            Assert.AreEqual("July 2nd", DateTimeHelper.GetFormattedMonthAndDay(Convert.ToDateTime("7/2/2007")));
            Assert.AreEqual("July 3rd", DateTimeHelper.GetFormattedMonthAndDay(Convert.ToDateTime("7/3/2007")));
        }

        [Test]
        public void TestGetDateDayWithSuffix()
        {
            Assert.AreEqual("1st", DateTimeHelper.GetDateDayWithSuffix(Convert.ToDateTime("7/1/2007")));
            Assert.AreEqual("2nd", DateTimeHelper.GetDateDayWithSuffix(Convert.ToDateTime("7/2/2007")));
            Assert.AreEqual("3rd", DateTimeHelper.GetDateDayWithSuffix(Convert.ToDateTime("7/3/2007")));
        }

        public void MethodName()
        {
            //System.Data.Sql.SqlDataSourceEnumerator sqlEnum = 
            System.Data.DataTable dt = System.Data.Sql.SqlDataSourceEnumerator.Instance.GetDataSources();

            bool server = false;
            bool instance = false;
            bool cluster = false;
            bool version = false;

            foreach (System.Data.DataRow dr in dt.Rows)
            {
                if (dr[0].ToString() != string.Empty) server = true;
                if (dr[1].ToString() != string.Empty) instance = true;
                if (dr[2].ToString() != string.Empty) cluster = true;
                if (dr[3].ToString() != string.Empty) version = true;

            }

            Assert.IsTrue(cluster);
            Assert.IsTrue(server);
            Assert.IsTrue(instance);
            Assert.IsTrue(version);

            

        }

    }
}
