using System;
using System.Text.RegularExpressions;
using NUnit.Framework;

namespace nToolbox.Core.Test
{
    [TestFixture]
    public class TestNumberHelper
    {
        /// <summary>
        /// Tests the function NumberHelper.SecondsToHourMinuteSeconds
        /// </summary>
        [Test]
        public void TestSecondsToHourMinuteSeconds()
        {
            Assert.AreEqual("00:00:03", NumberHelper.SecondsToHourMinuteSeconds(3));
            Assert.AreEqual("00:00:34", NumberHelper.SecondsToHourMinuteSeconds(34));
            Assert.AreEqual("00:01:34", NumberHelper.SecondsToHourMinuteSeconds(94));
            Assert.AreEqual("00:15:34", NumberHelper.SecondsToHourMinuteSeconds(934));
            Assert.AreEqual("01:34:53", NumberHelper.SecondsToHourMinuteSeconds(5693));
            Assert.AreEqual("41:24:53", NumberHelper.SecondsToHourMinuteSeconds(149093));
        }

        [Test]
        public void TestIsNumeric()
        {
            Assert.AreEqual(true, NumberHelper.IsNumeric("348343"));
            Assert.AreEqual(false, NumberHelper.IsNumeric("349dk390"));
            Assert.AreEqual(true, NumberHelper.IsNumeric("3432433"));
        }

        [Test]
        public void TestConvertStringArrayToIntegerArray()
        {
            // Assert.AreEqual()
        }

        [Test]
        public void TestIsNaturalNumber()
        {
            Assert.IsTrue(NumberHelper.IsNaturalNumber("548"));
            Assert.IsFalse(NumberHelper.IsNaturalNumber("0"));
            Assert.IsTrue(NumberHelper.IsNaturalNumber("1"));
            Assert.IsTrue(NumberHelper.IsNaturalNumber("2"));
        }

        [Test]
        public void TestIsWholeNumber()
        {
            Assert.IsFalse(NumberHelper.IsNaturalNumber("548.3432"));
            Assert.IsFalse(NumberHelper.IsNaturalNumber("1.34"));
            Assert.IsFalse(NumberHelper.IsNaturalNumber("0.349"));
            Assert.IsTrue(NumberHelper.IsNaturalNumber("34349"));
        }

        [Test]
        public void TestIsInteger()
        {
            string number1 = "12.3";
            string number2 = "123";
            string number3 = "Phil's a geek";

            Assert.IsFalse(NumberHelper.IsInteger(number1));
            Assert.IsTrue(NumberHelper.IsInteger(number2));
            Assert.IsFalse(NumberHelper.IsInteger(number3));
        }

        [Test]
        public void TestIsNumber()
        {
            string number1 = "12.3";
            string number2 = "123";
            string number3 = "Phil's a geek";

            Assert.IsTrue(NumberHelper.IsNumber(number1));
            Assert.IsTrue(NumberHelper.IsNumber(number2));
            Assert.IsFalse(NumberHelper.IsNumber(number3));
        }

        [Test]
        public void TestIsEven()
        {
            Assert.IsTrue(NumberHelper.IsEven(0));
            Assert.IsTrue(NumberHelper.IsEven(48));
            Assert.IsFalse(NumberHelper.IsEven(345));
        }

        [Test]
        public void TestIsOdd()
        {
            Assert.IsFalse(NumberHelper.IsOdd(0));
            Assert.IsFalse(NumberHelper.IsOdd(48));
            Assert.IsTrue(NumberHelper.IsOdd(345));
        }

        [Test]
        public void TestRandom()
        {
            // get a random number that should not contain zero
            int testint = NumberHelper.Random(true);
            //Regex reg = new Regex("^[^-][0]$");
            //Assert.IsTrue(reg.IsMatch(testint.ToString()));            
            Assert.IsTrue(NumberHelper.IsNumber(testint.ToString()));
            Assert.IsTrue(NumberHelper.IsNumber(NumberHelper.Random().ToString()));
            
            testint = NumberHelper.Random(1000);
            Assert.Less(testint, 1000);

            testint = NumberHelper.Random(50, 100);
            Assert.Less(testint, 100);
            Assert.Greater(testint, 50);
        }

    }
}
