using System;
using NUnit.Framework;

namespace nToolbox.Core.Test
{
	[TestFixture]
	public class TestStringHelper
	{
		[Test]
		public void TestAfter()
		{
			Assert.AreEqual("efg", StringHelper.After("abcdefg", "d"));
			Assert.AreEqual("bcdefg", StringHelper.After("abcdefg", "a"));
			Assert.AreEqual("g", StringHelper.After("abcdefg", "f"));
			Assert.AreEqual("", StringHelper.After("abcdefg", "g"));
			Assert.AreEqual("abcdefg", StringHelper.After("abcdefg", "h"));
			Assert.AreEqual("abcdefg", StringHelper.After("abcdefg", ""));

			Assert.AreEqual("efg", StringHelper.After("abcdefg", "cd"));

			ExpectException<ArgumentNullException>(
				"Null argument was accepted.",
				delegate { StringHelper.After("abcdefg", null); });

			ExpectException<ArgumentNullException>(
				"Null argument was accepted.",
				delegate { StringHelper.After(null, "b"); });
		}

		[Test]
		public void TestFirst()
		{
			Assert.AreEqual("", StringHelper.First(null, 9));
			Assert.AreEqual("a", StringHelper.First("abcdefg", 1));
			Assert.AreEqual("ab", StringHelper.First("abcdefg", 2));
			Assert.AreEqual("abc", StringHelper.First("abcdefg", 3));
			Assert.AreEqual("abcd", StringHelper.First("abcdefg", 4));
			Assert.AreEqual("abcde", StringHelper.First("abcdefg", 5));
			Assert.AreEqual("abcdef", StringHelper.First("abcdefg", 6));
			Assert.AreEqual("abcdefg", StringHelper.First("abcdefg", 7));
			Assert.AreEqual("abcdefg", StringHelper.First("abcdefg", 8));
			Assert.AreEqual("abcdefg", StringHelper.First("abcdefg", 22));
			Assert.AreEqual("", StringHelper.First("abcdefg", 0));

			ExpectException<ArgumentOutOfRangeException>(
				"Negative length was accepted.",
				delegate { StringHelper.First("abcdefg", -1); });
		}

		[Test]
		public void TestUpTo()
		{
			Assert.AreEqual("", StringHelper.UpTo("abc", "a"));
			Assert.AreEqual("a", StringHelper.UpTo("abc", "b"));
			Assert.AreEqual("ab", StringHelper.UpTo("abc", "c"));
			Assert.AreEqual("abc", StringHelper.UpTo("abc", "d"));
		}

		[Test]
		public void TestNullTrim()
		{
			Assert.AreEqual("abc", StringHelper.NullTrim("   abc"));
			Assert.AreEqual("abc", StringHelper.NullTrim("   abc   "));
			Assert.AreEqual("abc", StringHelper.NullTrim("abc   "));
			Assert.AreEqual("a b c", StringHelper.NullTrim("a b c"));
			Assert.AreEqual("", StringHelper.NullTrim(""));

			Assert.AreEqual("", StringHelper.NullTrim(null));

			Assert.AreEqual("1", StringHelper.NullTrim(new NullTrimData(1)));
			Assert.AreEqual("", StringHelper.NullTrim(new NullTrimData(2)));
			Assert.AreEqual("3", StringHelper.NullTrim(new NullTrimData(3)));
		}

		[Test]
		public void TestConcat()
		{
			Assert.AreEqual("1,2,3", StringHelper.Concat(",", 1, 2, 3));
			Assert.AreEqual("1,3", StringHelper.Concat(",", 1, null, 3));
			Assert.AreEqual("1,3", StringHelper.Concat(",", 1, "", 3));
		}

        [Test]
        public void TestLeft()
        {
            Assert.AreEqual("Testing This O", StringHelper.Left("Testing This Out", 14));
        }
        
        [Test]
        public void TestRight()
        {
            Assert.AreEqual("s Out", StringHelper.Right("Testing This Out", 5));
        }

        [Test]
        public void TestMid()
        {
            Assert.AreEqual("ng This", StringHelper.Mid("Testing This Out", 5, 7));
        }

        [Test]
        public void TestStringContainsNumber()
        {
            Assert.AreEqual(true, StringHelper.StringContainsNumber("349dkfei"));
            Assert.AreEqual(true, StringHelper.StringContainsNumber("34"));
            Assert.AreEqual(false, StringHelper.StringContainsNumber("kasdfiefkn askd jj"));
        }

        [Test]
        public void TestCapitalize()
        {
            Assert.AreEqual("This Should Be Capitalized", StringHelper.Capitalize("this should be capitalized"));
            Assert.AreEqual("Word", StringHelper.Capitalize("WORD"));
            Assert.AreEqual("", StringHelper.Capitalize(null));
        }
      
        /// <summary>
		/// Class used internally for testing NullTrim on an object that returns null from ToString.
		/// </summary>
		private class NullTrimData
		{
			private readonly int value;

			public NullTrimData(int value)
			{
				this.value = value;
			}

			public override string ToString()
			{
				if (value == 2) return null;
				else return "  " + value + " ";
			}
		}

		protected delegate void Operation();

		protected static void ExpectException<T>(string failureMessage, Operation action)
			where T: Exception
		{
			try
			{
				action();
			}
			catch (Exception ex)
			{
				if (ex.GetType() != typeof (T))
				{
					Assert.Fail(failureMessage);
				}
			}
		}
	}
}