using System.Collections.Generic;
using System.Web.UI.WebControls;
using nToolbox.Core.Test.UnitTestData;
using nToolbox.Web.Controls;
using NUnit.Framework;


namespace nToolbox.Core.Test.Web.Controls
{
    [TestFixture]
    public class DataBinderTest
    {
        [Test]
        public void Repeater_Binder_Test()
        {
            List<Person> people = SampleData.TestData();
            Repeater rpt = new Repeater();

            UIBinder
                .Bind(SampleData.TestData())
                .To(rpt)
                .Do();

            Assert.AreEqual(rpt.Items.Count, people.Count);
        }

        [Test]
        public void DropDownList_Binder_Test_Selected_Value()
        {
            List<Person> people = SampleData.TestData();
            DropDownList ddl = new DropDownList();

            UIBinder
                .Bind(people)
                .To(ddl)
                .InsertingAsFirstItem("blankValue", "blankText")
                .AutoSelectItemWithValueOf("Homer Simpson")
                .UsePropertyForDataValue("FullName")
                .UsePropertyForTextValue("FullName")
                .Do();

            
            Assert.AreEqual(ddl.Items.Count, people.Count+1);
            Assert.AreEqual(ddl.SelectedValue, "Homer Simpson");
        }

        [Test]
        public void DropDownList_Binder_Test_Selected_Text()
        {
            List<Person> people = SampleData.TestData();
            DropDownList ddl = new DropDownList();

            UIBinder
                .Bind(people)
                .To(ddl)
                .InsertingAsFirstItem("blankValue", "blankText")
                .AutoSelectItemWithTextOf("Homer")
                .UsePropertyForDataValue("FullName")
                .UsePropertyForTextValue("FirstName")
                .Do();


            Assert.AreEqual(ddl.Items.Count, people.Count + 1);
            Assert.AreEqual(ddl.SelectedItem.Text, "Homer");
        }
        
    }
}